﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using PIEHidDotNet;


namespace PIHidDotName_Csharp_Sample
{
    public partial class Form1 : Form, PIEDataHandler, PIEErrorHandler
    {
        PIEDevice[] devices;

        int[] cbotodevice = null; //for each item in the CboDevice list maps this index to the device index.  Max devices =100 
        byte[] wData = null; //write data buffer
        int selecteddevice = -1; //set to the index of CboDevice
        int singleLED = 0; //indicates whether single LED is running or not
        //for thread-safe way to call a Windows Forms control
        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        delegate void SetTextCallback(string text);
        Control c;
        //end thread-safe
       
        public Form1()
        {
            InitializeComponent();
        }

        private void BtnEnumerate_Click(object sender, EventArgs e)
        {
            CboDevices.Items.Clear();
            cbotodevice = new int[128];  //max # devices = 128
            //enumerate and setupinterfaces for all devices
            devices = PIEHidDotNet.PIEDevice.EnumeratePIE();
            if (devices.Length == 0)
            {
                toolStripStatusLabel1.Text = "No Devices Found";
            }
            else
            {
                //System.Media.SystemSounds.Beep.Play(); 
                int cbocount = 0; //keeps track of how many valid devices were added to the CboDevice box
                for (int i = 0; i < devices.Length; i++)
                {
                    //information about device
                    //PID = devices[i].Pid);
                    //HID Usage = devices[i].HidUsage);
                    //HID Usage Page = devices[i].HidUsagePage);
                    //HID Version = devices[i].Version);
                    if (devices[i].HidUsagePage == 0xc)
                    {
                        switch (devices[i].Pid)
                        {
                            case 693:
                                CboDevices.Items.Add("X-keys Stick (" + devices[i].Pid + "), ID: " + i);
                                cbotodevice[cbocount] = i;
                                cbocount++;
                                break;
                            default:
                                CboDevices.Items.Add("Unknown Device (" + devices[i].Pid + "), ID: " + i);
                                cbotodevice[cbocount] = i;
                                cbocount++;
                                break;
                        }
                        devices[i].SetupInterface();
                    }
                }
            }
            if (CboDevices.Items.Count > 0)
            {
                CboDevices.SelectedIndex = 0;
                selecteddevice = cbotodevice[CboDevices.SelectedIndex];
                wData = new byte[devices[selecteddevice].WriteLength];//go ahead and setup for write
            }

        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            //closeinterfaces on all devices
            for (int i = 0; i < CboDevices.Items.Count; i++)
            {
                devices[cbotodevice[i]].CloseInterface();
            }
            System.Environment.Exit(0);
        }

        private void BtnRed_Click(object sender, EventArgs e)
        {
            //write to currently selected device - turn on the Red LED
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength - 1; j++) //don't clear out last byte, the LED byte
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 186;
                wData[7] = (byte)(wData[7] | 128);

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success-Red LED";
                }
            }

        }

        private void BtnNoLEDS_Click(object sender, EventArgs e)
        {
            //write to currently selected device in CboDevice - turn off all LEDs
            if (CboDevices.SelectedIndex != -1)
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength; j++)
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 186;
                wData[7] = 0;
                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success - No LEDs";
                }
            }
        }

        private void BtnGreen_Click(object sender, EventArgs e)
        {
            //write to currently selected device - turn on the green LED
            if (CboDevices.SelectedIndex != -1)
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength - 1; j++) //don't clear out last byte, the LED byte
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 186;
                wData[7] = (byte)(wData[7] | 64);
                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success - Green LED";
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            toolStripStatusLabel1.Text = "";
        }

        private void BtnUnitID_Click(object sender, EventArgs e)
        {
            //Write Unit ID to currently selected device
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {
                //Program Switch must be set, check here
                byte[] data = null;
                int countout = 0;
                int ret = 1;
                while (ret != 0)
                {
                    ret = devices[selecteddevice].ReadLast(ref data);
                    countout++;
                    if (countout > 500)
                        break;

                }
                byte val2 = (byte)(data[6] & 16);
                if (val2 == 0)
                {
                    System.Media.SystemSounds.Beep.Play();
                    toolStripStatusLabel1.Text = "Set the Program Switch and try again";
                    return;
                }
                //end check of program switch position

                //write Unit ID given in the TxtSetUnitID box
                for (int j = 0; j < devices[selecteddevice].WriteLength; j++)
                {
                    wData[j] = 0;
                }

                wData[0] = 2;
                wData[1] = 189;
                wData[2] = (byte)(Convert.ToInt16(TxtSetUnitID.Text));

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success - Write Unit ID";
                }
            }
        }
        private void CboDevices_SelectedIndexChanged(object sender, EventArgs e)
        {
            selecteddevice = cbotodevice[CboDevices.SelectedIndex];
            wData = new byte[devices[selecteddevice].WriteLength];//size write array 
        }
        private void BtnCallback_Click(object sender, EventArgs e)
        {
            //setup callback if there are devices found for each device found

            if (CboDevices.SelectedIndex != -1)
            {
                //get initial state before turning on callback

                //read data
                byte[] data = null;
                while (0 == devices[selecteddevice].ReadData(ref data))
                {
                    //now handle data
                    byte val2 = (byte)(data[6] & 16);
                    if (val2 == 0)
                    {
                        LblSwitchPos.Text = "switch down";
                    }
                    else
                    {
                        LblSwitchPos.Text = "switch up";
                    }
                    LblUnitID.Text = data[5].ToString();
                    //write data to listbox
                    String output = "Polling: " + devices[selecteddevice].Pid + ", ID: " + selecteddevice.ToString() + ", data=";
                    for (int i = 0; i < 7; i++)
                    {
                        output = output + data[i].ToString() + " ";
                    }
                    //add only if different from last output

                    listBox1.Items.Add(output);
                    listBox1.SelectedIndex = listBox1.Items.Count - 1;
                }
                
                //end of get initial state
                
                for (int i = 0; i < CboDevices.Items.Count; i++)
                {
                    //use the cbotodevice array which contains the mapping of the devices in the CboDevices to the actual device IDs
                    devices[cbotodevice[i]].SetErrorCallback(this);
                    devices[cbotodevice[i]].SetDataCallback(this, DataCallbackFilterType.callOnChangedData);

                }

            }
        }
        //data callback    
        public void HandlePIEHidData(Byte[] data, PIEDevice sourceDevice)
        {
            //check the sourceDevice and make sure it is the same device as selected in CboDevice   
            if (sourceDevice == devices[selecteddevice])
            {
                if (data[0] == 2)
                {
                    //check the switch byte
                    byte val2 = (byte)(data[6] & 16);
                    if (val2 == 0)
                    {
                        c = this.LblSwitchPos;
                        this.SetText("switch down");

                    }
                    else
                    {
                        c = this.LblSwitchPos;
                        this.SetText("switch up");
                    }
                    //read the unit ID
                    c = this.LblUnitID;
                    this.SetText(data[5].ToString());
                }
                //write raw data to listbox1
                String output = "Callback: " + sourceDevice.Pid + ", ID: " + selecteddevice.ToString() + ", data=";
                for (int i = 0; i < 7; i++) //readlength is actually 32 but bytes after the 7th are not useful
                {
                    output = output + data[i].ToString() + " ";
                }
                this.SetListBox(output);

                //set the LEDs if SingleLED
                if (singleLED == 1)
                {
                    byte temp;
                    for (int j = 0; j < sourceDevice.WriteLength; j++)
                    {
                        wData[j] = 0;
                    }
                    wData[0] = 2;
                    wData[1] = 187;
                    temp = 0;

                    if ((byte)(data[1] & 1) != 0) temp = (byte)(temp + 1);
                    if ((byte)(data[2] & 1) != 0) temp = (byte)(temp + 2);
                    if ((byte)(data[3] & 1) != 0) temp = (byte)(temp + 4);
                    if ((byte)(data[4] & 1) != 0) temp = (byte)(temp + 8);
                    if ((byte)(data[1] & 2) != 0) temp = (byte)(temp + 16);
                    if ((byte)(data[2] & 2) != 0) temp = (byte)(temp + 32);
                    if ((byte)(data[3] & 2) != 0) temp = (byte)(temp + 64);
                    if ((byte)(data[4] & 2) != 0) temp = (byte)(temp + 128);


                    wData[7] = temp;
                    temp = 0;
                    if ((byte)(data[1] & 4) != 0) temp = (byte)(temp + 1);
                    if ((byte)(data[2] & 4) != 0) temp = (byte)(temp + 2);
                    if ((byte)(data[3] & 4) != 0) temp = (byte)(temp + 4);
                    if ((byte)(data[4] & 4) != 0) temp = (byte)(temp + 8);
                    if ((byte)(data[1] & 8) != 0) temp = (byte)(temp + 16);
                    if ((byte)(data[2] & 8) != 0) temp = (byte)(temp + 32);
                    if ((byte)(data[3] & 8) != 0) temp = (byte)(temp + 64);
                    if ((byte)(data[4] & 8) != 0) temp = (byte)(temp + 128);

                    wData[6] = temp;
                    sourceDevice.WriteData(wData);
                }


            }
        }
        //error callback
        public void HandlePIEHidError(Int32 error, PIEDevice sourceDevice)
        {
            this.SetToolStrip("Error: " + error.ToString());
        }
        //for threadsafe setting of Windows Forms control
        private void SetText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.c.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.c.Text = text;
            }
        }
        //for threadsafe setting of Windows Forms control
        private void SetListBox(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.listBox1.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetListBox);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.listBox1.Items.Add(text);
                this.listBox1.SelectedIndex = this.listBox1.Items.Count - 1;
            }
        }
        //for threadsafe setting of Windows Forms control
        private void SetToolStrip(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.statusStrip1.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetToolStrip);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.toolStripStatusLabel1.Text = text;
            }
        }

        private void BtnSetKey_Click(object sender, EventArgs e)
        {
            //For users of the dongle feature 
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {
                //Programming switch must be up (red). Users of this feature
                //may want to add a program switch check.
                //This routine is done once per unit by the developer prior to sale.

                //Program Switch must be set, check here
                byte[] data = null;
                int countout = 0;
                int ret = 1;
                while (ret != 0)
                {
                    ret = devices[selecteddevice].ReadLast(ref data);
                    countout++;
                    if (countout > 500)
                        break;

                }
                byte val2 = (byte)(data[6] & 16);
                if (val2 == 0)
                {
                    System.Media.SystemSounds.Beep.Play();
                    toolStripStatusLabel1.Text = "Set the Program Switch and try again";
                    return;
                }
                //end check of program switch position

                //Pick 4 numbers between 1 and 254.
                int K0 = 7;    //pick any number between 1 and 254, 0 and 255 not allowed
                int K1 = 58;   //pick any number between 1 and 254, 0 and 255 not allowed
                int K2 = 33;   //pick any number between 1 and 254, 0 and 255 not allowed
                int K3 = 243;  //pick any number between 1 and 254, 0 and 255 not allowed
                //Save these numbers, they are needed to check the key!

                //Write these to the device
                wData[0] = 2;
                wData[1] = 192; // 0xC0; //set key command
                wData[2] = (byte)K0;
                wData[3] = (byte)K1;
                wData[4] = (byte)K2;
                wData[5] = (byte)K3;
                wData[6] = 0;
                wData[7] = 0;

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success-Set Dongle Key";
                }
            }
        }

        private void BtnCheckKey_Click(object sender, EventArgs e)
        {
            //This is done within the developer's application to check for the correct
            //hardware.  The K0-K3 values must be the same as those entered in Set Key.
            LblPassFail.Text = "";
            if (CboDevices.SelectedIndex != -1)
            {

                //check hardware
                //randomn numbers
                int N0 = 3;   //pick any number between 1 and 254
                int N1 = 1;   //pick any number between 1 and 254
                int N2 = 4;   //pick any number between 1 and 254
                int N3 = 1;   //pick any number between 1 and 254

                //this is the key from set key
                int K0 = 7;
                int K1 = 58;
                int K2 = 33;
                int K3 = 243;

                //hash and save these for comparison later
                int R0;
                int R1;
                int R2;
                int R3;
                PIEDevice.DongleCheck(K0, K1, K2, K3, N0, N1, N2, N3, out R0, out R1, out R2, out R3);

                wData[0] = 2;
                wData[1] = 193; // 0xC1;
                wData[2] = (byte)N0;
                wData[3] = (byte)N1;
                wData[4] = (byte)N2;
                wData[5] = (byte)N3;
                wData[6] = 0;
                wData[7] = 0;
                devices[selecteddevice].ClearBuffer();
                int result = devices[selecteddevice].WriteData(wData);

                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success-Check Dongle Key";
                }

                //after this write the next read beginning with byte1=3 will give 4 values which are used below for comparison

                byte[] data = null;
                int countout = 0;
                int ret = devices[selecteddevice].BlockingReadData(ref data, 100);
                while ((ret == 0 && data[0] != 3) || ret == 304)
                {
                    if (ret == 304)
                    {
                        // Didn't get any data for 100ms, increment the countout extra
                        countout += 100;
                    }
                    countout++;
                    if (countout > 500) //increase this if have to check more than once
                        break;
                    ret = devices[selecteddevice].BlockingReadData(ref data, 100);
                }

                //bytes 2-10 will be 'Dongle Key' also

                if (ret == 0 && data[0] == 3)
                {
                    bool fail = false;
                    if (R0 != data[11]) fail = true;
                    if (R1 != data[12]) fail = true;
                    if (R2 != data[13]) fail = true;
                    if (R3 != data[14]) fail = true;
                    if (fail == false)
                    {
                        LblPassFail.Text = "Pass-Correct Hardware Found";
                    }
                    else
                    {
                        LblPassFail.Text = "Fail-Correct Hardward Not Found";
                    }
                }


            }

        }

        private void BtnBLOn_Click(object sender, EventArgs e)
        {
            //write to currently selected device - turn on the backlighting
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength; j++)
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 187;
                wData[6] = 0xFF;
                wData[7] = 0xFF;

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success-Backlighting On";
                }
            }
        }

        private void BtnBLOff_Click(object sender, EventArgs e)
        {
            //write to currently selected device - turn off the backlighting
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength; j++)
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 187;
                wData[6] = 0;
                wData[7] = 0;

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    toolStripStatusLabel1.Text = "Write Success-Backlighting Off";
                }
            }
        }

        private void BtnSingleLED_Click(object sender, EventArgs e)
        {
            if (singleLED == 0)
            {
                singleLED = 1;
                BtnSingleLED.Text = "Single LED Running";
            }
            else
            {
                singleLED = 0;
                BtnSingleLED.Text = "Start Single LED";
            }
        }

        private void LblPassFail_Click(object sender, EventArgs e)
        {

        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            listBox1.Items.Clear();
           
        }

        private void BtnPolling_Click(object sender, EventArgs e)
        {
            if (BtnPolling.Text == "Start Polling")
            {
                timer1.Enabled = true;
                BtnPolling.Text = "Stop Polling";
            }
            else
            {
                timer1.Enabled = false;
                BtnPolling.Text = "Start Polling";
            }


        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //read data
            byte[] data = null;
            while (0 == devices[selecteddevice].ReadData(ref data))
            {
                    //now handle data
                    byte val2 = (byte)(data[6] & 16);
                    if (val2 == 0)
                    {
                        LblSwitchPos.Text = "switch down";
                    }
                    else
                    {
                        LblSwitchPos.Text = "switch up";
                    }
                    LblUnitID.Text = data[5].ToString();
                    //write data to listbox
                    String output = "Polling: " + devices[selecteddevice].Pid + ", ID: " + selecteddevice.ToString() + ", data=";
                    for (int i = 0; i < 7; i++)
                    {
                        output = output + data[i].ToString() + " ";
                    }
                    //add only if different from last output

                    listBox1.Items.Add(output);
                    listBox1.SelectedIndex = listBox1.Items.Count - 1;         
            }
        }
    }
    
}
